@@ -32,6 +32,7 @@ import openerplib
 import psycopg2
 import logging
 import decimal
+import time
 import sys, traceback
 _logger = logging.getLogger(__name__)
 
@@ -80,6 +81,10 @@ class oer_etl(object):
             self.__jobs[job_id] = res
         return res
     
+    def get_job_state(self, job_id):
+        job_obj = self.local.get_model('etl.job')
+        return job_obj.read([job_id])[0]['state']
+    
     def get_server(self, server_id):
         res = False
         if self.__servers.has_key(server_id):
@@ -117,6 +122,9 @@ class oer_etl(object):
             if type(query_begin) is unicode and job['query_encoding']:
                 query_begin = query_begin.encode(job['query_encoding'])
             cr.execute(query_begin)
+            if job.get('query_begin_delay'):
+                self.log(job['id'],'Query Begin is executing, time to waiting %d sec. (%s)'%(job['query_begin_delay'],time.strftime('%Y-%m-%d %H:%M.%S')))
+                time.sleep(job['query_begin_delay'])
         query = job['query']%localdict
         if type(query) is unicode and job['query_encoding']:
             query = query.encode(job['query_encoding'])
@@ -150,6 +158,8 @@ class oer_etl(object):
     
     def get_resolve_xml_id(self, job_id, xml_id):
         res = False
+        if type(xml_id) is not str and type(xml_id) is not unicode:
+            return res
         job = self.get_job(job_id)
         mod_xml = xml_id.split('.')
         if len(mod_xml) == 2:
@@ -167,7 +177,7 @@ class oer_etl(object):
             line_obj = self.local.get_model('etl.mapping.line')
             mapping = mapping_obj.read([mapping_id])[0]
             job = self.get_job(job_id)
-            self.__value_mapping[mapping_id] = {}
+            self.__value_mapping[mapping_id] = {'__return_null_value__': mapping.get('return_null',True)}
             for line in line_obj.read(mapping['line_ids']):
                 line_value = False
                 if line.get('map_ref'):
@@ -178,8 +188,15 @@ class oer_etl(object):
                     line_value = self.get_resolve_xml_id(job_id, line['map_xml_id'])
                 elif line.get('map_char'):
                     line_value = line['map_char']
+                if line.get('is_default',False):
+                    self.__value_mapping[mapping_id]['__is_default_value__'] = line_value
                 self.__value_mapping[mapping_id][line['name']] = line_value
-        val = self.__value_mapping[mapping_id].get(val,val)
+        if not val and self.__value_mapping[mapping_id]['__return_null_value__']:
+            return val
+        if self.__value_mapping[mapping_id].has_key('__is_default_value__') and not self.__value_mapping[mapping_id].has_key(val):
+            val = self.__value_mapping[mapping_id].get('__is_default_value__')
+        else:
+            val = self.__value_mapping[mapping_id].get(val,val)
         return val
         
     def get_values(self, job_id, row):
@@ -189,15 +206,18 @@ class oer_etl(object):
             val = eval(map['value'],row)
             if map['mapping_id']:
                 val = self.get_value_mapping(map['mapping_id'][0],val,job_id)
+            row['__value_mapping__'] = val
             if map['field_type'] in ('char','text','selection'):
-                if type(val) is not unicode and job['query_encoding']:
+                if val and type(val) is not unicode and job['query_encoding']:
                     val = val.decode(job['query_encoding'])
             elif map['field_type'] == 'date':
-                if type(val) is not str or type(val) is not unicode:
+                if val and type(val) not in (str, unicode):
                     val = val.strftime('%Y-%m-%d')
             elif map['field_type'] == 'datetime':
-                if type(val) is not str or type(val) is not unicode:
+                if val and (type(val) is not str or type(val) is not unicode):
                     val = val.strftime('%Y-%m-%d %H:%M:%S')
+            elif map['search_null'] and (not val):
+                pass
             elif map['field_type'] == 'many2one':
                 val_obj = self.get_connection(job['dst_server_id'][0]).get_model(map['field_relation'])
                 if map['name_search']:
@@ -257,7 +277,9 @@ class oer_etl(object):
         try:
             new_id = load_model.create(values)
         except Exception as e:
-            self.log(job['id'],'Unexpected Error: \n%s\n\n%s'%(e.faultCode,e.faultString), stack=sys.exc_info(), level='error', pk=pk)
+            msg = '\n%s.create '%load_model.model_name + str(values)
+            msg += '\nUnexpected Error: \n%s\n\n%s'%(e.faultCode,e.faultString)
+            self.log(job['id'], msg, stack=sys.exc_info(), level='error', pk=pk)
         else:
             self.log(job['id'],'Ok', id=new_id, pk=pk)
         return new_id
@@ -280,6 +302,6 @@ class oer_etl(object):
             exc_type, exc_value, exc_traceback = stack
             stack = traceback.format_exception(exc_type, exc_value, exc_traceback)
             vals['traceback'] = ''.join(stack)
-        if self.log_print: print to_print
+        if self.log_print: print to_print.encode('ascii','replace')
         return log_obj.create(vals)
     
\ No newline at end of file
